/*
 * Useful CLIPS functions declared for C++
 *
 */

#ifndef clipscpp
#define clipscpp

#ifdef __WATCOMC__
#define VOID_ARG void
struct dataObject
  {
   VOID *supplementalInfo;
   int type;
   VOID *value;
   int begin;
   int end;
   struct dataObject *next;
  };

struct expr 
   {
    short type;
    VOID *value;
    struct expr *argList;
    struct expr *nextArg;
   };
   
#define arg_list argList
#define next_arg nextArg

typedef struct expr EXPRESSION;

typedef struct exprHashNode
  {
   unsigned hashval;
   unsigned count;
   struct expr *exp;
   struct exprHashNode *nxt;
   long bsaveID;
  } EXPRESSION_HN;

/************************************************************/
/* symbolHashNode STRUCTURE:                                */
/************************************************************/
struct symbolHashNode
  {
   struct symbolHashNode *next;
   int count;
   int depth;
   unsigned int markedEphemeral : 1;
   unsigned int neededSymbol : 1;
   unsigned int bucket : 14;
   char *contents;
  };
  
/************************************************************/
/* floatHashNode STRUCTURE:                                  */
/************************************************************/
struct floatHashNode
  {
   struct floatHashNode *next;
   int count;
   int depth;
   unsigned int markedEphemeral : 1;
   unsigned int neededFloat : 1;
   unsigned int bucket : 14;
   double contents;
  };

/************************************************************/
/* integerHashNode STRUCTURE:                               */
/************************************************************/
struct integerHashNode
  {
   struct integerHashNode *next;
   int count;
   int depth;
   unsigned int markedEphemeral : 1;
   unsigned int neededInteger : 1;
   unsigned int bucket : 14;
   long int contents;
  };

typedef struct dataObject DATA_OBJECT;
typedef struct dataObject * DATA_OBJECT_PTR;

#define GetType(target)         ((target).type) 
#define GetDOLength(target)       (((target).end - (target).begin) + 1)
#define GetpDOLength(target)      (((target)->end - (target)->begin) + 1)
#define GetDOBegin(target)        ((target).begin + 1)
#define GetDOEnd(target)          ((target).end + 1)
#define GetpDOBegin(target)       ((target)->begin + 1)
#define GetpDOEnd(target)         ((target)->end + 1)
#define SetDOBegin(target,val)   ((target).begin = (val) - 1) 
#define SetDOEnd(target,val)     ((target).end = (val) - 1)
#define SetpDOBegin(target,val)   ((target)->begin = (val) - 1) 
#define SetpDOEnd(target,val)     ((target)->end = (val) - 1)
#define ClipsGetValue(target)        ((target).value) 
#define GetpValue(target)       ((target)->value) 

#define DOPToString(target) (((struct symbolHashNode *) (target->value))->contents)
#define DOPToDouble(target) (((struct floatHashNode *) (target->value))->contents)
#define DOPToFloat(target) ((float) (((struct floatHashNode *) (target->value))->contents))
#define DOPToLong(target) (((struct integerHashNode *) (target->value))->contents)
#define DOPToInteger(target) ((int) (((struct integerHashNode *) (target->value))->contents))
#define DOPToPointer(target)       ((target)->value) 

#define DOToString(target) (((struct symbolHashNode *) (target.value))->contents)
#define DOToDouble(target) (((struct floatHashNode *) (target.value))->contents)
#define DOToFloat(target) ((float) (((struct floatHashNode *) (target.value))->contents))
#define DOToLong(target) (((struct integerHashNode *) (target.value))->contents)
#define DOToInteger(target) ((int) (((struct integerHashNode *) (target.value))->contents))
#define DOToPointer(target)        ((target).value) 

#define ValueToString(target) (((struct symbolHashNode *) (target))->contents)
#define ValueToDouble(target) (((struct floatHashNode *) (target))->contents)
#define ValueToLong(target) (((struct integerHashNode *) (target))->contents)
#define ValueToInteger(target) ((int) (((struct integerHashNode *) (target))->contents))
#define ValueToBitMap(target) ((VOID *) ((struct bitMapHashNode *) (target))->contents)

#define SetpType(target,val)    ((target)->type = (val)) 
#define SetpValue(target,val)   ((target)->value = (VOID *) (val)) 

struct field
  {
   short int type;
   VOID *value;
  };
    
struct multifield
  {
   unsigned busyCount;
   short depth;
   unsigned short multifieldLength;
   struct multifield *next;
   struct field theFields[1];
  };

typedef struct multifield SEGMENT;
typedef struct multifield * SEGMENT_PTR;
typedef struct multifield * MULTIFIELD_PTR;
typedef struct field FIELD;
typedef struct field * FIELD_PTR;

#define GetMFLength(target)     (((struct multifield *) (target))->multifieldLength) 
#define GetMFPtr(target,index)  (&(((struct multifield *) (target))->theFields[index-1])) 
#define SetMFType(target,index,value)  (((struct multifield *) (target))->theFields[index-1].type = (value)) 
#define SetMFValue(target,index,val)  (((struct multifield *) (target))->theFields[index-1].value = (VOID *) (val))  
#define GetMFType(target,index)  (((struct multifield *) (target))->theFields[index-1].type) 
#define GetMFValue(target,index)  (((struct multifield *) (target))->theFields[index-1].value) 

#include "constant.h"
#define PTIF (int (*)(VOID_ARG))

#else
struct dataObject;
#endif

// #define CLIPS6

#if (!defined(CLIPS5) && !defined(CLIPS6))
#define CLIPS5
#endif

#ifndef WXCLIPS
#define WXCLIPS
#endif

extern "C" {
int CLIPSFunctionCall(char *, char *, struct dataObject *);

#ifdef CLIPS6
Bool AddRouter(char *,int,int (*)(char *),
               int (*)(char *,char *),
               int (*)(char *),
               int (*)(int,char *),
               int (*)(int));
#else
Bool AddRouter(char *,int,int (*)(),int (*)(),int (*)(),int (*)(),int (*)());
#endif
void *AddSymbol(char *s);
int  ArgCountCheck(char *, int, int);
void *AssertString(char *);
int  Bload(char *);
int  Bsave(char *);
void *CreateMultifield(int);
int  DefineFunction(char *, int, int (*)(void), char *);
int  DefineFunction2(char *, int, int (*)(void), char *, char *); // added by Bob Orchard
void ExecuteBatchFile(char *); // Added by JACS
char *GetDefruleName(void *);
void *GetNextDefrule(void *);
void *FindDefrule(char *);
void InitializeCLIPS(void);

#ifdef CLIPS6
void SetBreak(void *);
// int  AddRunFunction (char *, auto void (*)(), int);           //needs fixing for 6 RHW
int  AddRunFunction (char *, void (*)(void), int);           //needs fixing for 6 RHW
void Clear(void);
void ListDeffacts (char *, void *);             //modified for 6 RobW
void ShowBreaks(char *, void *);              //replaces listbreakpoints for 6 RobW
void ListDefrules (char *, void *);             //modified for 6 RobW
void Agenda(char *logicalName, void *theModule); //added for 6 RobW
void ListDeftemplates (char *, void *);         //modified for 6 RobW
void ListDefglobals (char *, void *);           //modified for 6 RobW
int  Load(char *);                               // modified for 6 RobW 
int  RemoveBreak(void *);                       //modified for 6 by RobW
long int  Run (long int);                           //modified of 6.  RHW
int  Save(char *);                                  //replaces saveconstructs RW.
int Watch (char *);          // added (replacing setwatch and addwatch. RobW
int Unwatch (char *);       // added (replacing setwatch and addwatch. RobW
void Reset(void);
#else
void AddBreakpoint(void *);
int  AddRunFunction(char *, int (*)(), int);
int  AddWatchItem(char *item, int *flag, int priority);
void ClearCLIPS(void);
void ListBreakpoints(void);
void ListDeffacts(void);
void ListDefrules(void);
void ListAgenda(void);
void ListDeftemplates(void);
void ListDefglobals(void);
int  LoadConstructs(char *);
int  RemoveBreakpoint(void *);
void RunCLIPS(int);
int  SaveConstructs(char *);
int  SetWatchItem(char *item, Bool activate);
void ResetCLIPS(void);
#endif

void SetEvaluationError(Bool);
void SetHaltExecution(Bool);
int  ListMatches(void *);
int  OpenBatch(char *, int);
void PeriodicCleanup(Bool cleanupAllDepths,Bool useHeuristics);
int  PrintCLIPS(char *, char *);

Bool RouteCommand(char *);
int  RtnArgCount();
double RtnDouble(int);
char *RtnLexeme(int);
long RtnLong(int);
struct dataObject *RtnUnknown(int, dataObject *);
void SetMultifieldErrorValue(dataObject *);
void SetPrintWhileLoading(Bool);
void UserFunctions(void);

#ifdef wx_msw
#ifndef WIN32
int  system(char *);
#endif
#endif


}

#ifdef CLIPS6
#define RunCLIPS(x) Run(x)
#define LoadConstructs(x) Load(x)
#define RemoveBreakpoint(x) RemoveBreak(x)
#define AddBreakpoint(x) SetBreak(x)
#define ListBreakpoints() ShowBreaks("stdout",NULL)
#define SaveConstructs(x) Save(x)
#define ListAgenda() Agenda("stdout",NULL)
#define ClearCLIPS() Clear()
#endif

#endif // clipscpp
