/*
 * File:	wxclipsx.cc
 * Purpose:	Client for talking to wxCLIPS by DDE (Windows only, as yet)
 * Author:	Julian Smart
 * Created:	1995
 * Updated:	
 * Copyright:	(c) 1995, AIAI, University of Edinburgh
 */

static const char sccsid[] = "%W% %G%";

#include "wx.h"
#include "wx_ipc.h"
#include "wxclipsx.h"

MyFrame *frame = NULL;

// This statement initialises the whole application
MyApp myApp;

char ipc_buffer[4000];
wxListBox *the_list = NULL;

MyConnection *the_connection = NULL;
MyClient *my_client ;

// The `main program' equivalent, creating the windows and returning the
// main frame
wxFrame *MyApp::OnInit(void)
{
  wxIPCInitialize();
  char *server = "WXCLIPS";
  char hostName[256];
  wxGetHostName(hostName, sizeof(hostName));
  char *host = hostName;
  char *command = NULL;

  int i = 1;
  while (i < argc)
  {
    if ((strcmp(argv[i], "-c") == 0))
    {
      i ++;
      wxBuffer[0] = 0;
      // Copy the rest of the command line into wxBuffer as the command to execute
      while (i < argc)
      {
        strcat(wxBuffer, argv[i]);
        i ++;
        if (i < argc)
          strcat(wxBuffer, " ");
      }
      command = copystring(wxBuffer);
    }
    else if (strcmp(argv[i], "-h") == 0)
    {
      i ++;
      if (i < argc)
      {
        host = copystring(argv[i]);
        i ++;
      }
    }
    else if (strcmp(argv[i], "-s") == 0)
    {
      i ++;
      if (i < argc)
      {
        server = copystring(argv[i]);
        i ++;
      }
    }
    else
    {
      wxMessageBox("Unrecognised switch", "Error");
      i = argc;
    }
  }

  // Create a new client
  my_client = new MyClient;
  the_connection = (MyConnection *)my_client->MakeConnection(host, server, "WXCLIPS");

  if (!the_connection)
  {
    wxMessageBox("Could not connect to wxCLIPS", "wxCLIPS client error");
    return NULL;
  }

  if (command)
  {
    ExecuteClipsCommand(command);
    the_connection->Disconnect();
    delete my_client;
    wxIPCCleanUp();
    return NULL;
  }

  // Create the main frame window
  frame = new MyFrame(NULL, "wxCLIPS Client", 400, 0, 300, 200);

  // Give it an icon
#ifdef wx_msw
  wxIcon *icon = new wxIcon("aiai_icn");
#endif
#ifdef wx_x
  wxIcon *icon = new wxIcon("aiai.xbm");
#endif

  frame->SetIcon(icon);

  // Make a menubar
  wxMenu *file_menu = new wxMenu;

  file_menu->Append(CLIENT_EXECUTE, "&Execute a wxCLIPS command");
//  file_menu->Append(CLIENT_REQUEST, "&Request");
//  file_menu->Append(CLIENT_POKE, "&Poke");
  file_menu->Append(CLIENT_QUIT, "E&xit");

  wxMenuBar *menu_bar = new wxMenuBar;

  menu_bar->Append(file_menu, "&File");

  // Associate the menu bar with the frame
  frame->SetMenuBar(menu_bar);

  // Make a panel
  frame->panel = new wxPanel(frame);

  frame->Show(TRUE);

  // Essential - return the main frame window
  return frame;
}

// Define my frame constructor
MyFrame::MyFrame(wxFrame *frame, char *title, int x, int y, int w, int h):
  wxFrame(frame, title, x, y, w, h)
{
  panel = NULL;
}

// Intercept menu commands
void MyFrame::OnMenuCommand(int id)
{
  switch (id)
  {
    case CLIENT_EXECUTE:
    {
      if (the_connection)
      {
        char *command = wxGetTextFromUser("Enter a wxCLIPS DDE command");
        if (command)
        {
          command = copystring(command);
          ExecuteClipsCommand(command);
          delete[] command;
        }
      }
      break;
    }
    case CLIENT_POKE:
    {
      if (the_connection)
        the_connection->Poke("An item", "Some data to poke at the server!");
      break;
    }
    case CLIENT_REQUEST:
    {
      if (the_connection)
      {
        char *data = the_connection->Request("An item");
        if (data)
          wxMessageBox(data, "Client: Request", wxOK);
      }
      break;
    }
    case CLIENT_QUIT:
    {
      OnClose();
      if (my_client)
        delete my_client ;
      wxIPCCleanUp() ;
      delete this;
      break;
    }
  }
}

// Define the behaviour for the frame closing
Bool MyFrame::OnClose(void)
{
  if (the_connection)
  {
    the_connection->Disconnect();
  }
  return TRUE;
}

MyClient::MyClient(void)
{
}

wxConnection *MyClient::OnMakeConnection(void)
{
  return new MyConnection;
}

MyConnection::MyConnection(void):wxConnection(ipc_buffer, 3999)
{
}

MyConnection::~MyConnection(void)
{
  the_connection = NULL;
}

Bool MyConnection::OnAdvise(char *topic, char *item, char *data, int size, int format)
{
  if (the_list)
  {
    int n = the_list->FindString(data);
    if (n > -1)
      the_list->SetSelection(n);
  }
  return TRUE;
}

void ExecuteClipsCommand(char *command)
{
  if (the_connection)
  {
    the_connection->Execute(command);
  }
}

