/*
 * wxcodbc.cc
 * wxWindows ODBC commands.
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#if defined(wx_msw) && USE_ODBC && WXODBC

#include "wx_hash.h"
#include "wx_types.h"
#include "wx_ipc.h"

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wx_cmds.h"
#include "wxcmdsp.h"

#ifdef USE_CLIPS
#ifndef __WATCOMC__
#include "clips.h"
#endif
#endif

/************************************************************************
 * Database commands
 *
 ************************************************************************/

#ifdef USE_CLIPS
long clipsDatabaseCreate()
{
  if (ArgCountCheck("clipsDatabaseCreate", EXACTLY, 0) == -1)
    return 0;

  return wxcDatabaseCreate();
}

long clipsDatabaseOpen()
{
  if (ArgCountCheck("clipsDatabaseOpen", AT_LEAST, 2) == -1)
    return 0;

  WXID id = RtnLong(1);
  char *source = RtnLexeme(2);
  Bool exclusive = FALSE;
  Bool readOnly = TRUE;
  char *username = "ODBC";
  char *password = "";

  if (RtnArgCount() > 2)
    exclusive = (Bool)RtnLong(3);
  if (RtnArgCount() > 3)
    readOnly = (Bool)RtnLong(4);
  if (RtnArgCount() > 4)
    username = RtnLexeme(5);
  if (RtnArgCount() > 5)
    password = RtnLexeme(6);
  
  return wxcDatabaseOpen(id, source, exclusive, readOnly, username, password);
}

long clipsDatabaseClose()
{
  if (ArgCountCheck("clipsDatabaseClose", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return wxcDatabaseClose(id);
}

long clipsDatabaseIsOpen()
{
  if (ArgCountCheck("clipsDatabaseIsOpen", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return wxcDatabaseIsOpen(id);
}

char *clipsDatabaseGetErrorCode()
{
  if (ArgCountCheck("clipsDatabaseGetErrorCode", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  WXID id = RtnLong(1);
  char *s = wxcDatabaseGetErrorCodeString(id);
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

char *clipsDatabaseGetDatabaseName()
{
  if (ArgCountCheck("clipsDatabaseGetDatabaseName", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  WXID id = RtnLong(1);
  char *s = wxcDatabaseGetDatabaseName(id);
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

char *clipsDatabaseGetErrorMessage()
{
  if (ArgCountCheck("clipsDatabaseGetErrorMessage", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  WXID id = RtnLong(1);
  char *s = wxcDatabaseGetErrorMessage(id);
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

long clipsDatabaseErrorOccurred()
{
  if (ArgCountCheck("clipsDatabaseErrorOccurred", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long) wxcDatabaseErrorOccurred(id);
}

long clipsDatabaseGetErrorNumber()
{
  if (ArgCountCheck("clipsDatabaseGetErrorNumber", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long) wxcDatabaseGetErrorNumber(id);
}

long clipsDatabaseDelete()
{
  if (ArgCountCheck("clipsDatabaseDelete", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long) wxcDatabaseDelete(id);
}

// Recordset
long clipsRecordSetCreate()
{
  if (ArgCountCheck("clipsRecordSetCreate", AT_LEAST, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  char *openType = "wxOPEN_TYPE_DYNASET";
  char *options = "wxOPTION_DEFAULT";

  if (RtnArgCount() > 1)
    openType = RtnLexeme(2);
  if (RtnArgCount() > 2)
    options = RtnLexeme(3);
  
  return (long) wxcRecordSetCreate(id, openType, options);
}

long clipsRecordSetDelete()
{
  if (ArgCountCheck("clipsRecordSetDelete", AT_LEAST, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long) wxcRecordSetDelete(id);
}

long clipsRecordSetQuery()
{
  if (ArgCountCheck("clipsRecordSetQuery", AT_LEAST, 2) == -1)
    return 0;

  WXID id = RtnLong(1);
  char *cols = RtnLexeme(2);
  char *table = NULL;
  char *filter = NULL;
  if (RtnArgCount() > 2)
    table = RtnLexeme(3);
  if (RtnArgCount() > 3)
    filter = RtnLexeme(4);
    
  return (long) wxcRecordSetQuery(id, cols, table, filter);
}

long clipsRecordSetGetNumberFields()
{
  if (ArgCountCheck("clipsRecordSetGetNumberFields", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long) wxcRecordSetGetNumberFields(id);
}

long clipsRecordSetGetNumberParams()
{
  if (ArgCountCheck("clipsRecordSetGetNumberParams", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long) wxcRecordSetGetNumberParams(id);
}

long clipsRecordSetGetNumberRecords()
{
  if (ArgCountCheck("clipsRecordSetGetNumberRecords", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long) wxcRecordSetGetNumberRecords(id);
}

long clipsRecordSetGetNumberCols()
{
  if (ArgCountCheck("clipsRecordSetGetNumberCols", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long) wxcRecordSetGetNumberCols(id);
}

char *clipsRecordSetGetFilter()
{
  if (ArgCountCheck("clipsRecordSetGetFilter", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  WXID id = RtnLong(1);
  char *s = wxcRecordSetGetFilter(id);
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

char *clipsRecordSetGetSortString()
{
  if (ArgCountCheck("clipsRecordSetGetSortString", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  WXID id = RtnLong(1);
  char *s = wxcRecordSetGetSortString(id);
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

long clipsRecordSetGetDatabase()
{
  if (ArgCountCheck("clipsRecordSetGetDatabase", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return wxcRecordSetGetDatabase(id);
}

char *clipsRecordSetGetErrorCode()
{
  if (ArgCountCheck("clipsRecordSetGetErrorCode", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  WXID id = RtnLong(1);
  char *s = wxcRecordSetGetErrorCodeString(id);
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

long clipsRecordSetGetResultSet()
{
  if (ArgCountCheck("clipsRecordSetGetResultSet", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetGetResultSet(id);
}

long clipsRecordSetExecuteSql()
{
  if (ArgCountCheck("clipsRecordSetExecuteSql", EXACTLY, 2) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetExecuteSql(id, RtnLexeme(2));
}

long clipsRecordSetGetTables()
{
  if (ArgCountCheck("clipsRecordSetGetTables", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetGetTables(id);
}

long clipsRecordSetGetColumns()
{
  if (ArgCountCheck("clipsRecordSetGetColumns", AT_LEAST, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  char *table = NULL;
  if (RtnArgCount() > 1)
    table = RtnLexeme(2);
  return (long)wxcRecordSetGetColumns(id, table);
}

#if wxVERSION_NUMBER > 3
long clipsRecordSetGetPrimaryKeys()
{
  if (ArgCountCheck("clipsRecordSetGetPrimaryKeys", AT_LEAST, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  char *table = NULL;
  if (RtnArgCount() > 1)
    table = RtnLexeme(2);
  return (long)wxcRecordSetGetPrimaryKeys(id, table);
}

long clipsRecordSetGetForeignKeys()
{
  if (ArgCountCheck("clipsRecordSetGetForeignKeys", AT_LEAST, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  char *table = NULL;
  if (RtnArgCount() > 1)
    table = RtnLexeme(2);
  char *ftable = NULL;
  if (RtnArgCount() > 2)
    ftable = RtnLexeme(3);
  return (long)wxcRecordSetGetForeignKeys(id, table, ftable);
}
#endif

char *clipsRecordSetGetTableName()
{
  if (ArgCountCheck("clipsRecordSetTableName", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  WXID id = RtnLong(1);
  char *s = wxcRecordSetGetTableName(id);
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

long clipsRecordSetSetTableName()
{
  if (ArgCountCheck("clipsRecordSetTableName", EXACTLY, 2) == -1)
    return 0;

  WXID id = RtnLong(1);
  char *table = RtnLexeme(2);
  return (long)wxcRecordSetSetTableName(id, table);
}

long clipsRecordSetIsOpen()
{
  if (ArgCountCheck("clipsRecordSetIsOpen", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetIsOpen(id);
}

long clipsRecordSetIsBof()
{
  if (ArgCountCheck("clipsRecordSetIsBof", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetIsBOF(id);
}

long clipsRecordSetIsEof()
{
  if (ArgCountCheck("clipsRecordSetIsEof", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetIsEOF(id);
}

long clipsRecordSetIsDeleted()
{
  if (ArgCountCheck("clipsRecordSetIsDeleted", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetIsDeleted(id);
}

long clipsRecordSetGetIntData()
{
  if (ArgCountCheck("clipsRecordSetGetIntData", EXACTLY, 2) == -1)
    return 0;

  WXID id = RtnLong(1);

  // Second argument can be a string column name or integer column position.
  DATA_OBJECT temp;
  RtnUnknown(2, &temp);

  switch (ClipsGetType(temp))
  {
    case INTEGER:
    {
      int col = (int)DOToLong(temp);
      return wxcRecordSetGetIntData(id, col);
    }
    case SYMBOL:
    case STRING:
    {
      char *col = DOToString(temp);
      return wxcRecordSetGetIntData(id, col);
    }
    default:
    {
      return 0;
    }
  }
  return 0;
}

double clipsRecordSetGetFloatData()
{
  if (ArgCountCheck("clipsRecordSetGetFloatData", EXACTLY, 2) == -1)
    return 0.0;

  WXID id = RtnLong(1);

  // Second argument can be a string column name or integer column position.
  DATA_OBJECT temp;
  RtnUnknown(2, &temp);

  switch (ClipsGetType(temp))
  {
    case INTEGER:
    {
      int col = (int)DOToLong(temp);
      return wxcRecordSetGetFloatData(id, col);
    }
    case SYMBOL:
    case STRING:
    {
      char *col = DOToString(temp);
      return wxcRecordSetGetFloatData(id, col);
    }
    default:
    {
      return 0.0;
    }
  }
  return 0.0;
}

char *clipsRecordSetGetCharData()
{
  if (ArgCountCheck("clipsRecordSetGetCharData", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  WXID id = RtnLong(1);

  // Second argument can be a string column name or integer column position.
  DATA_OBJECT temp;
  RtnUnknown(2, &temp);

  switch (ClipsGetType(temp))
  {
    case INTEGER:
    {
      int col = (int)DOToLong(temp);
      char *s = wxcRecordSetGetCharData(id, col);
      if (s)
        return (char *)AddSymbol(s);
      else
        return (char *)AddSymbol("");
    }
    case SYMBOL:
    case STRING:
    {
      char *col = DOToString(temp);
      char *s = wxcRecordSetGetCharData(id, col);
      if (s)
        return (char *)AddSymbol(s);
      else
        return (char *)AddSymbol("");
    }
    default:
    {
      return 0;
    }
  }
  return 0;
}

char *clipsRecordSetGetColName()
{
  if (ArgCountCheck("clipsRecordSetGetColName", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  WXID id = RtnLong(1);
  char *s = wxcRecordSetGetColName(id, (int)RtnLong(2));
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

char *clipsRecordSetGetColType()
{
  if (ArgCountCheck("clipsRecordSetGetColType", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  WXID id = RtnLong(1);

  // Second argument can be a string column name or integer column position.
  DATA_OBJECT temp;
  RtnUnknown(2, &temp);

  switch (ClipsGetType(temp))
  {
    case INTEGER:
    {
      int col = (int)DOToLong(temp);
      char *s = wxcRecordSetGetColTypeString(id, col);
      if (s)
        return (char *)AddSymbol(s);
      else
        return (char *)AddSymbol("");
    }
    case SYMBOL:
    case STRING:
    {
      char *col = DOToString(temp);
      char *s = wxcRecordSetGetColTypeString(id, col);
      if (s)
        return (char *)AddSymbol(s);
      else
        return (char *)AddSymbol("");
    }
    default:
    {
      return 0;
    }
  }
  return 0;
}

long clipsRecordSetMove()
{
  if (ArgCountCheck("clipsRecordSetMove", EXACTLY, 2) == -1)
    return 0;

  WXID id = RtnLong(1);
  long rows = RtnLong(2);
  return (long)wxcRecordSetMove(id, rows);
}

long clipsRecordSetMoveFirst()
{
  if (ArgCountCheck("clipsRecordSetMoveFirst", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetMoveFirst(id);
}

long clipsRecordSetMoveLast()
{
  if (ArgCountCheck("clipsRecordSetMoveLast", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetMoveLast(id);
}

long clipsRecordSetMoveNext()
{
  if (ArgCountCheck("clipsRecordSetMoveNext", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetMoveNext(id);
}

long clipsRecordSetMovePrev()
{
  if (ArgCountCheck("clipsRecordSetMovePrev", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetMovePrev(id);
}

long clipsRecordSetGoto()
{
  if (ArgCountCheck("clipsRecordSetGoto", EXACTLY, 2) == -1)
    return 0;

  WXID id = RtnLong(1);
  long row = RtnLong(2);
  return (long)wxcRecordSetGoto(id, row);
}

long clipsRecordSetGetDataSources()
{
  if (ArgCountCheck("clipsRecordSetGetDataSources", EXACTLY, 1) == -1)
    return 0;

  WXID id = RtnLong(1);
  return (long)wxcRecordSetGetDataSources(id);
}

long clipsRecordSetIsFieldDirty()
{
  if (ArgCountCheck("clipsRecordSetIsFieldDirty", EXACTLY, 2) == -1)
    return FALSE;

  WXID id = RtnLong(1);

  // Second argument can be a string column name or integer column position.
  DATA_OBJECT temp;
  RtnUnknown(2, &temp);

  switch (ClipsGetType(temp))
  {
    case INTEGER:
    {
      int col = (int)DOToLong(temp);
      return (long) wxcRecordSetIsFieldDirty(id, col);
    }
    case SYMBOL:
    case STRING:
    {
      char *col = DOToString(temp);
      return (long) wxcRecordSetIsFieldDirty(id, col);
    }
    default:
    {
      return FALSE;
    }
  }
  return FALSE;
}

long clipsRecordSetIsFieldNull()
{
  if (ArgCountCheck("clipsRecordSetIsFieldNull", EXACTLY, 2) == -1)
    return FALSE;

  WXID id = RtnLong(1);

  // Second argument can be a string column name or integer column position.
  DATA_OBJECT temp;
  RtnUnknown(2, &temp);

  switch (ClipsGetType(temp))
  {
    case INTEGER:
    {
      int col = (int)DOToLong(temp);
      return (long) wxcRecordSetIsFieldNull(id, col);
    }
    case SYMBOL:
    case STRING:
    {
      char *col = DOToString(temp);
      return (long) wxcRecordSetIsFieldNull(id, col);
    }
    default:
    {
      return FALSE;
    }
  }
  return FALSE;
}

long clipsRecordSetIsColNullable()
{
  if (ArgCountCheck("clipsRecordSetIsColNullable", EXACTLY, 2) == -1)
    return FALSE;

  WXID id = RtnLong(1);

  // Second argument can be a string column name or integer column position.
  DATA_OBJECT temp;
  RtnUnknown(2, &temp);

  switch (ClipsGetType(temp))
  {
    case INTEGER:
    {
      int col = (int)DOToLong(temp);
      return (long) wxcRecordSetIsColNullable(id, col);
    }
    case SYMBOL:
    case STRING:
    {
      char *col = DOToString(temp);
      return (long) wxcRecordSetIsColNullable(id, col);
    }
    default:
    {
      return FALSE;
    }
  }
  return FALSE;
}

long clipsRecordSetSetFieldDirty()
{
  if (ArgCountCheck("clipsRecordSetSetFieldDirty", AT_LEAST, 2) == -1)
    return FALSE;

  WXID id = RtnLong(1);

  Bool dirty = TRUE;
  if (RtnArgCount() > 2)
    dirty = (Bool)RtnLong(3);

  // Second argument can be a string column name or integer column position.
  DATA_OBJECT temp;
  RtnUnknown(2, &temp);

  switch (ClipsGetType(temp))
  {
    case INTEGER:
    {
      int col = (int)DOToLong(temp);
      return (long) wxcRecordSetSetFieldDirty(id, col, dirty);
    }
    case SYMBOL:
    case STRING:
    {
      char *col = DOToString(temp);
      return (long) wxcRecordSetSetFieldDirty(id, col);
    }
    default:
    {
      return FALSE;
    }
  }
  return FALSE;
}


#endif

/*************************************************************/
/* UserFunctions:  The function which informs CLIPS of any   */
/*   user defined functions.  In the default case, there are */
/*   no user defined functions.  To define functions, either */
/*   this function must be replaced by a function with the   */
/*   same name within this file, or this function can be     */
/*   deleted from this file and included in another file.    */
/*   User defined functions may be included in this file or  */
/*   other files.                                            */
/*   Example of redefined UserFunctions:                     */
/*     UserFunctions()                                       */
/*       {                                                   */
/*        DefineFunction("fun1",'i',fun1,"fun1");            */
/*        DefineFunction("other",'f',other,"other");         */
/*       }                                                   */
/*************************************************************/

#ifdef USE_CLIPS

void wxODBCUserFunctions()
{  
  DefineFunction("database-create", 'l', PTIF clipsDatabaseCreate, "clipsDatabaseCreate");
  DefineFunction("database-open", 'l', PTIF clipsDatabaseOpen, "clipsDatabaseOpen");
  DefineFunction("database-close", 'l', PTIF clipsDatabaseClose, "clipsDatabaseClose");
  DefineFunction("database-is-open", 'l', PTIF clipsDatabaseIsOpen, "clipsDatabaseIsOpen");
  DefineFunction("database-get-error-code", 's', PTIF clipsDatabaseGetErrorCode, "clipsDatabaseGetErrorCode");
  DefineFunction("database-get-database-name", 's', PTIF clipsDatabaseGetDatabaseName, "clipsDatabaseGetDatabaseName");
  DefineFunction("database-get-error-message", 's', PTIF clipsDatabaseGetErrorMessage, "clipsDatabaseGetErrorMessage");
  DefineFunction("database-error-occurred", 'l', PTIF clipsDatabaseErrorOccurred, "clipsDatabaseErrorOccurred");
  DefineFunction("database-get-error-number", 'l', PTIF clipsDatabaseGetErrorNumber, "clipsDatabaseGetErrorNumber");
  DefineFunction("database-delete", 'l', PTIF clipsDatabaseDelete, "clipsDatabaseDelete");

  DefineFunction("recordset-create", 'l', PTIF clipsRecordSetCreate, "clipsRecordSetCreate");
  DefineFunction("recordset-delete", 'l', PTIF clipsRecordSetDelete, "clipsRecordSetDelete");
  DefineFunction("recordset-query", 'l', PTIF clipsRecordSetQuery, "clipsRecordSetQuery");
  DefineFunction("recordset-get-number-fields", 'l', PTIF clipsRecordSetGetNumberFields, "clipsRecordSetGetNumberFields");
  DefineFunction("recordset-get-number-params", 'l', PTIF clipsRecordSetGetNumberParams, "clipsRecordSetGetNumberParams");
  DefineFunction("recordset-get-number-records", 'l', PTIF clipsRecordSetGetNumberRecords, "clipsRecordSetGetNumberRecords");
  DefineFunction("recordset-get-number-cols", 'l', PTIF clipsRecordSetGetNumberCols, "clipsRecordSetGetNumberCols");
  DefineFunction("recordset-get-filter", 's', PTIF clipsRecordSetGetFilter, "clipsRecordSetGetFilter");
  DefineFunction("recordset-get-sort-string", 's', PTIF clipsRecordSetGetSortString, "clipsRecordSetGetSortString");
  DefineFunction("recordset-get-database", 'l', PTIF clipsRecordSetGetDatabase, "clipsRecordSetGetDatabase");
  DefineFunction("recordset-get-error-code", 's', PTIF clipsRecordSetGetErrorCode, "clipsRecordSetGetErrorCode");
  DefineFunction("recordset-get-result-set", 'l', PTIF clipsRecordSetGetResultSet, "clipsRecordSetGetResultSet");
  DefineFunction("recordset-execute-sql", 'l', PTIF clipsRecordSetExecuteSql, "clipsRecordSetExecuteSql");
  DefineFunction("recordset-get-tables", 'l', PTIF clipsRecordSetGetTables, "clipsRecordSetGetTables");
  DefineFunction("recordset-get-columns", 'l', PTIF clipsRecordSetGetColumns, "clipsRecordSetGetColumns");
#if wxRELEASE_NUMBER > 3
  DefineFunction("recordset-get-primary-keys", 'l', PTIF clipsRecordSetGetPrimaryKeys, "clipsRecordSetGetPrimaryKeys");
  DefineFunction("recordset-get-foreign-keys", 'l', PTIF clipsRecordSetGetForeignKeys, "clipsRecordSetGetForeignKeys");
#endif
  DefineFunction("recordset-get-table-name", 's', PTIF clipsRecordSetGetTableName, "clipsRecordSetGetTableName");
  DefineFunction("recordset-set-table-name", 'l', PTIF clipsRecordSetSetTableName, "clipsRecordSetSetTableName");
  DefineFunction("recordset-is-open", 'l', PTIF clipsRecordSetIsOpen, "clipsRecordSetIsOpen");
  DefineFunction("recordset-is-bof", 'l', PTIF clipsRecordSetIsBof, "clipsRecordSetIsBof");
  DefineFunction("recordset-is-eof", 'l', PTIF clipsRecordSetIsEof, "clipsRecordSetIsEof");
  DefineFunction("recordset-is-deleted", 'l', PTIF clipsRecordSetIsDeleted, "clipsRecordSetIsDeleted");
  DefineFunction("recordset-get-int-data", 'l', PTIF clipsRecordSetGetIntData, "clipsRecordSetGetIntData");
  DefineFunction("recordset-get-float-data", 'd', PTIF clipsRecordSetGetFloatData, "clipsRecordSetGetFloatData");
  DefineFunction("recordset-get-char-data", 's', PTIF clipsRecordSetGetCharData, "clipsRecordSetGetCharData");
  DefineFunction("recordset-get-col-name", 's', PTIF clipsRecordSetGetColName, "clipsRecordSetGetColName");
  DefineFunction("recordset-get-col-type", 's', PTIF clipsRecordSetGetColType, "clipsRecordSetGetColType");
  DefineFunction("recordset-move", 'l', PTIF clipsRecordSetMove, "clipsRecordSetMove");
  DefineFunction("recordset-move-first", 'l', PTIF clipsRecordSetMoveFirst, "clipsRecordSetMoveFirst");
  DefineFunction("recordset-move-last", 'l', PTIF clipsRecordSetMoveLast, "clipsRecordSetMoveLast");
  DefineFunction("recordset-move-next", 'l', PTIF clipsRecordSetMoveNext, "clipsRecordSetMoveNext");
  DefineFunction("recordset-move-prev", 'l', PTIF clipsRecordSetMovePrev, "clipsRecordSetMovePrev");
  DefineFunction("recordset-goto", 'l', PTIF clipsRecordSetGoto, "clipsRecordSetGoto");
  DefineFunction("recordset-get-data-sources", 'l', PTIF clipsRecordSetGetDataSources, "clipsRecordSetGetDataSources");
  DefineFunction("recordset-is-field-dirty", 'l', PTIF clipsRecordSetIsFieldDirty, "clipsRecordSetIsFieldDirty");
  DefineFunction("recordset-is-field-null", 'l', PTIF clipsRecordSetIsFieldNull, "clipsRecordSetIsFieldNull");
  DefineFunction("recordset-is-col-nullable", 'l', PTIF clipsRecordSetIsColNullable, "clipsRecordSetIsColNullable");
  DefineFunction("recordset-set-field-dirty", 'l', PTIF clipsRecordSetSetFieldDirty, "clipsRecordSetSetFieldDirty");

}

#endif


#endif
  // ODBC
